﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
if not nUI then nUI = {}; end
if not nUI_Unit then nUI_Unit = {}; end
if not nUI_DefaultConfig then nUI_DefaultConfig = {}; end

local CheckInteractDistance = CheckInteractDistance;
local CreateFrame           = CreateFrame;
local GetNetStats           = GetNetStats;
local GetTime               = GetTime;
local GetTalentTabInfo      = GetTalentTabInfo;
local NotifyInspect         = NotifyInspect;

-------------------------------------------------------------------------------

nUI_TalentBuild        = {};

local first_queue      = true;
local updating_queue   = 0;
local queue_check      = 0;
local num_queued       = 0;
local TalentBuildCache = {};
local TalentBuildQueue = {};
local QueueCount       = {};
local talent_events    = CreateFrame( "Frame", "nUI_TalentBuildEvents", WorldFrame );

-------------------------------------------------------------------------------
-- add a request to the talent build inspection queue (API entry point)

function nUI_GetTalentBuild( unit_info, id, callback )

	updating_queue    = updating_queue+1;
	QueueCount[id]    = (QueueCount[id] or 0)+1;
	num_queued        = num_queued+1;

	if first_queue then		
--		nUI_Config.TalentLog = {};
		first_queue = false;
	end
	
	--table.insert( nUI_Config.TalentLog, "queued "..id.." at ["..num_queue.."] for GUID "..(unit.GUID or "<nil>") );
	
	TalentBuildQueue[num_queued] = 
	{
		ready    = false,
		pending  = false,
		unit     = unit_info,
		guid     = unit_info.guid,
		id       = id,
		callback = callback,
		expires  = 0;
	};
		
	updating_queue = updating_queue-1;
	
end

-------------------------------------------------------------------------------
-- remove the first item from the talent build inspection queue

local function Dequeue( queue )
	
	if num_queued > 0 then

		--table.insert( nUI_Config.TalentLog, "dequeued "..queue.id.." for GUID "..(queue.guid or "<nil>") );

		unit = queue.unit;

		if QueueCount[queue.id] > 0 then
			QueueCount[queue.id] = QueueCount[queue.id]-1;
		end
		
		num_queued = num_queued-1;

		for i=1,num_queued do
			TalentBuildQueue[i] = TalentBuildQueue[i+1];
		end				
	end	
end

-------------------------------------------------------------------------------
-- move a pending talent build inspection to the end of the queue

local function Requeue( queue )
	
	if num_queued > 0 then
		
		--table.insert( nUI_Config.TalentLog, "requeing "..queue.id.." for GUID "..(queue.guid or "<nil>") );
		
		local unit = queue.unit;
		local id   = queue.id;
		
		Dequeue( queue );
		nUI_GetTalentBuild( unit, id );
		
	end
end

-------------------------------------------------------------------------------
-- read the talent table

local function ReadTalents( queue, unit )

	local id        = queue.id;
	local guid      = queue.guid;
	local is_player = unit and unit.is_self or false;
	local match     = unit and unit.guid == guid or false;
	local build     = {};
	
	if match then

		--table.insert( nUI_Config.TalentLog, "reading "..id.." for GUID "..(guid or "<nil>") );
	
		local remoteInspect = not is_player;
		
		build.name1, build.iconTexture1, build.num1, build.background1  = GetTalentTabInfo( 1, remoteInspect );
		build.name2, build.iconTexture2, build.num2, build.background2  = GetTalentTabInfo( 2, remoteInspect );
		build.name3, build.iconTexture3, build.num3, build.background3  = GetTalentTabInfo( 3, remoteInspect );

		-- make sure that the read we are doing is for the unit we intend it to be for
	
		if build.name1 == nil or build.name2 == nil or build.name3 == nil then
			build = nil;
			
		elseif build.num1 == nil or build.num2 == nil or build.num3 == nil then
			build = nil;
			
		else
			if build.num1 > 0 or build.num2 > 0 or build.num3 > 0 then
				--table.insert( nUI_Config.TalentLog, "cached GUID "..(guid or "<nil>") );
				TalentBuildCache[guid] = build;
			end
		end
	end
	
	if QueueCount[id] == 0 then
		--table.insert( nUI_Config.TalentLog, "updating "..id.." for GUID "..(guid or "<nil>") );
		queue.callback( id, queue.unit, match and build or nil );
	end	
end

-------------------------------------------------------------------------------
-- flag the unit spec ready for reading on a unit we're waiting for

local function onTalentBuildEvent()
	
	if num_queued > 0 then
	
		local queue = TalentBuildQueue[1];
		
		--table.insert( nUI_Config.TalentLog, "queued "..queue.id.." for GUID "..(queue.guid or "<nil>") );
			
		queue.ready = true;
		
	end	
end

talent_events:SetScript( "OnEvent", onTalentBuildEvent );

-------------------------------------------------------------------------------
-- process the talent build inspection queue

local function onTalentBuildUpdate( who, elapsed )
	
	if num_queued > 0 then
			
		queue_check = queue_check + elapsed;
		
		-- check the queue at 12.5 fps for expirations
		
		if updating_queue == 0
		and queue_check > 0.08 
		then
		
			queue_check = 0;

			for i=1,num_queued do			
				
				-- get this first item on the queue
				
				local queue       = TalentBuildQueue[1];			
				local id          = queue.id;
				local unit        = queue.unit;
				local unit_info   = nUI_Unit:getUnitInfo( id );
				local is_player   = unit_info and unit_info.is_self or false;
	
				-- if the pending talent build ready is ready, get it now
				
				if queue.ready then
		
					Dequeue( queue );
					ReadTalents( queue, unit_info );
					
				-- if it is not already pending processing, start it now
	
				elseif not queue.pending then
	
					local cache       = unit.guid and TalentBuildCache[unit.guid] or nil;
					local _,_,latency = GetNetStats();
	
					queue.pending = true;
					queue.expires = GetTime() + latency + 1;
					
					-- if this is the player, run it locally now
					
					if is_player 
					then 
					
						Dequeue( queue );
						ReadTalents( queue, unit_info );
					
					-- if the unit no longer exists or it is no longer player controlled
					
					elseif not unit_info
					or not unit_info.is_player
					then

						--table.insert( nUI_Config.TalentLog, "invalid queue unit "..queue.id.." for GUID "..(queue.guid or "<nil>") );
					
						Dequeue( queue );
	
						if QueueCount[id] == 0 then
							queue.callback( id, unit );
						end
	
					-- if we've already fetched this GUID's talent build, use the cached copy
					
					elseif cache then
						
						--table.insert( nUI_Config.TalentLog, "using cache for "..queue.id.." for GUID "..(queue.guid or "<nil>") );
						
						Dequeue( queue );
						
						if QueueCount[id] == 0 then
							queue.callback( id, unit, cache );
						end
						
					-- if the unit is out of range, requeue it to try again later
					
					elseif not unit_info.is_visble
					or not CheckInteractDistance( id, 1 ) then

						--table.insert( nUI_Config.TalentLog, "out of range for "..queue.id.." for GUID "..(queue.guid or "<nil>") );
					
						Requeue( queue );
	
						if num_queued == 1 then break; end
	
					-- otherwise, intiate a new unit inspection and we're done
					
					else

						--table.insert( nUI_Config.TalentLog, "inspecting "..queue.id.." for GUID "..(queue.guid or "<nil>") );
		
						NotifyInspect( id );
						break;
						
					end
	
				-- if the unit is pending and no longer exists, we can drop the request
				
				elseif not unit_info
				or not unit_info.is_player
				then

					--table.insert( nUI_Config.TalentLog, "invalid pending unit "..queue.id.." for GUID "..(queue.guid or "<nil>") );
				
					Dequeue( queue );
	
					if QueueCount[id] == 0 then
						queue.callback( id, unit );
					end
					
				-- if the unit has moved out of range, we can requeue and try again later

				elseif not unit_info.is_visible
				or not CheckInteractDistance( id, 1 ) then

					--table.insert( nUI_Config.TalentLog, "out of range pending unit "..queue.id.." for GUID "..(queue.guid or "<nil>") );
			
					Requeue( queue );
					
					if num_queued == 1 then break; end
			
				-- or if it's expired, give it up
				
				elseif GetTime() > queue.expires then

					--table.insert( nUI_Config.TalentLog, "expired pending unit "..queue.id.." for GUID "..(queue.guid or "<nil>") );
				
					Requeue( queue );
					
					if num_queued == 1 then break; end
				
				end
			end
		end
	end
end

talent_events:SetScript( "OnUpdate", onTalentBuildUpdate );
talent_events:RegisterEvent( "INSPECT_TALENT_READY" );
